
#ifndef FIXMATH_H
#define FIXMATH_H

/****************************************************************************/

#define FIXED_ONE       INT_TO_FIXED(1)
#define FIXED_PI        205887L
#define FIXED_2PI       411775L
#define FIXED_E         178144L
#define FIXED_ROOT2      74804L
#define FIXED_ROOT3     113512L
#define FIXED_GOLDEN    106039L

#define INT_TO_FIXED(x)         ((x) << 16)
#define DOUBLE_TO_FIXED(x)      ((Fixed32)(x * 65536.0 + 0.5))
#define FIXED_TO_INT(x)         ((x) >> 16)
#define FIXED_TO_DOUBLE(x)      (((double)x) / 65536.0)
#define ROUND_FIXED_TO_INT(x)   (((x) + 0x8000) >> 16)

#define ANG90		(FIXED_PI/2)
#define ANG180		(FIXED_PI)
#define ANG270		(ANG180+ANG90)
#define ANG360		(FIXED_2PI)

/****************************************************************************/

typedef long	Fixed32;

Fixed32 FixSqrt( Fixed32 );
Fixed32 FixSqrti(unsigned long x);
//Fixed32 FixSinCos( Fixed32 theta, Fixed32 *sinus );
Fixed32 FixAtan2( Fixed32, Fixed32 );

/****************************************************************************/
	
__inline static Fixed32 FixMul(Fixed32 eins,Fixed32 zwei)
{	
	return (Fixed32) ( (double) eins * (double) zwei * 0.0000152587890625 );
}

/****************************************************************************/

__inline static Fixed32 FixDiv(Fixed32 eins,Fixed32 zwei)
{	
	eins = (double) eins / (double) zwei * 65536.0;

	return eins;
}

/****************************************************************************/

__inline static long FixSqr( Fixed32 a )
{
	a = INT_TO_FIXED( a );
	return a * a;
}	

/****************************************************************************/

__inline static Fixed32 FixSinCos( Fixed32 theta, Fixed32 *sinus )
{
	#define MAX_TRIG	1024
	extern Fixed32 SinCosTab[];
    theta = FIXED_TO_INT( FixMul( theta << 9, 20861 ) );
    theta &= (MAX_TRIG - 1);
    
    *sinus = SinCosTab[theta];
    
    if( ( theta += 256 ) >= MAX_TRIG )
	theta = theta - MAX_TRIG;
		
    return SinCosTab[theta];
}	

/****************************************************************************/

#endif
